/* ----------------------------------------------------------------------- */
/*
 * Program:
 * Demonstrate deadlock
 *
 * Purpose:
 * Use Intel Thread Checker to identify threading errors
 *
 * Original WinAPI source code:
 * Intel(R) Thread Checker for Windows
 *
 * Modifications:
 * Alexander Pozdneev
 */
/* ----------------------------------------------------------------------- */
#include <stdio.h>
#include <pthread.h>
/* ----------------------------------------------------------------------- */
int globalX = 0;
int globalY = 0;
/* ----------------------------------------------------------------------- */
pthread_mutex_t m0, m1;
/* ----------------------------------------------------------------------- */
static void *work0(void *arg)
{
    pthread_mutex_lock(&m0);
        globalX++;
        pthread_mutex_lock(&m1);
            globalY++;
        pthread_mutex_unlock(&m1);
    pthread_mutex_unlock(&m0);

    return 0;
}
/* ----------------------------------------------------------------------- */
static void *work1(void *arg)
{
    pthread_mutex_lock(&m1);
        globalX++;
        pthread_mutex_lock(&m0);
            globalY++;
        pthread_mutex_unlock(&m0);
    pthread_mutex_unlock(&m1);

    return 0;
}
/* ----------------------------------------------------------------------- */
int main (int argc, char *argv[])
{
    pthread_t h0, h1;

    pthread_mutex_init(&m0, NULL);
    pthread_mutex_init(&m1, NULL);

    printf("START\n");

    pthread_create(&h0, NULL, work0, NULL);
    pthread_create(&h1, NULL, work1, NULL);

    printf("TOTAL = (%d,%d)\n", globalX, globalY);

    pthread_join(h0, NULL);
    pthread_join(h1, NULL);

    printf("STOP\n");

    pthread_mutex_destroy(&m0);
    pthread_mutex_destroy(&m1);

    return 0;
}
/* ----------------------------------------------------------------------- */
